library ieee;
use ieee.std_logic_1164.all;

package config is

  -- resets entities on next rising edge of clock
  -- (RESET must stay asserted until this happens)
  CONSTANT USE_SYNC_RESET : boolean := FALSE;
  -- resets on falling edge of RESET
  CONSTANT USE_ASYNC_RESET : boolean := NOT(USE_SYNC_RESET);
  
  function sync_reset(rst  : std_logic) return boolean;
  function async_reset(rst : std_logic) return boolean; 

  type targettechs is 
  (gen, virtex, virtex2, atc35, atc25, atc18, fs90, umc18, tsmc25, proasic, axcel);
 
  constant TARGET_TECH : targettechs := gen;
  constant INFER_RAM : boolean := false;
  constant INFER_PADS : boolean := false;
  constant INFER_PCI_PADS : boolean := false;
  
end config;

package body config is

  ----------------------------------------------------------------------------
  -- generic reset functions definitions
  ----------------------------------------------------------------------------
  function sync_reset(rst : std_logic) return boolean is
  begin
    if (USE_SYNC_RESET) then
      return (rst = '1');
    else
      return false;
    end if;
  end;

  function async_reset(rst : std_logic) return boolean is
  begin
    if (USE_ASYNC_RESET) then
      return (rst = '1');
    else
      return false;
    end if;
  end; 

end config;
