library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

use work.syslog.all;
use work.crc_pack.all;

entity crc_pack_tb is
end crc_pack_tb;

architecture beh of crc_pack_tb is

  subtype byte is integer range 0 to 255;
  type byte_vector is array (natural range <>) of byte;

  constant MOD_RPAT : byte_vector := (
    16#BE#, 16#D7#, 16#23#, 16#47#, 16#6B#, 16#8F#,
    16#B3#, 16#14#, 16#5E#, 16#FB#, 16#35#, 16#59#);

begin  -- beh

  test: process

    function flipvector (vector : std_logic_vector) return std_logic_vector is
      variable return_vector : std_logic_vector (vector'high downto vector'low );
    begin
      for i in vector'low to vector'high loop
        return_vector(vector'high - i + vector'low) := vector(i);
      end loop;
      return return_vector;
    end flipvector;
    
    variable frame : byte_vector(0 to MOD_RPAT'length * 29 + 4 - 1);
    variable crc : std_logic_vector(31 downto 0);
    
  begin -- process test

    -- Short random pattern
    -- Taken from ieee 802.3-2000 Annex 36A
    for i in MOD_RPAT'range loop
      for j in 0 to 28 loop
        frame(j * MOD_RPAT'length + i) := MOD_RPAT(i);
      end loop;  -- j
    end loop;  -- i
    -- add CRC field
    frame(348 to 351) := (16#2F#, 16#E0#, 16#AA#, 16#EF#); 

    -- invert first data bits equals loading "111111...1"
    crc := (others => '1');
    -- step through the frame
    for i in frame'range loop
      crc := next_crc(crc,
                      flipvector(conv_std_logic_vector(frame(i),8)),
                      CRC32_GEN);
    end loop;  -- i
    
    -- compare the remainder
    if crc = "11000111000001001101110101111011" then
      syslog(note, "CRC ok");
    else
      syslog(error, "Bad CRC");
    end if;
    
    syslog_terminate;
    wait;
    
  end process test;

end beh;
